@@ -38,7 +38,6 @@
 #include "screen.h"
 #include "resize.h"
 #include "add_window.h"
-#include "siconify.xbm"
 #include <X11/Xos.h>
 #include <X11/Xmu/CharSet.h>
 
@@ -59,9 +58,26 @@ int iconmgr_iconx = 0, iconmgr_textx = 0;
 
 WList *Active = NULL;
 WList *DownIconManager = NULL;
+
+/* was an external file - djhjr - 10/30/02 */
+#define siconify_width 11
+#define siconify_height 11
+/*
+static unsigned char siconify_bits[] = {
+   0xff, 0x07, 0x01, 0x04, 0x0d, 0x05, 0x9d, 0x05, 0xb9, 0x04, 0x51, 0x04,
+   0xe9, 0x04, 0xcd, 0x05, 0x85, 0x05, 0x01, 0x04, 0xff, 0x07};
+*/
+
 int iconifybox_width = siconify_width;
 int iconifybox_height = siconify_height;
 
+/* djhjr - 10/30/02 */
+void SetIconMgrPixmap(filename)
+char *filename;
+{
+	Scr->iconMgrIconName = filename;
+}
+
 /***********************************************************************
  *
  *  Procedure:
@@ -77,6 +93,45 @@ int iconifybox_height = siconify_height;
  ***********************************************************************
  */
 
+struct Colori {
+    Pixel color;
+    Pixmap pix;
+    struct Colori *next;
+};
+
+#if 0
+Pixmap Create3DIconManagerIcon (cp)
+ColorPair cp;
+{
+    unsigned int w, h;
+    struct Colori *col;
+    static struct Colori *colori = NULL;
+
+    w = (unsigned int) siconify_width;
+    h = (unsigned int) siconify_height;
+
+    for (col = colori; col; col = col->next) {
+	if (col->color == cp.back) break;
+    }
+    if (col != NULL) return (col->pix);
+    col = (struct Colori*) malloc (sizeof (struct Colori));
+    col->color = cp.back;
+    col->pix   = XCreatePixmap (dpy, Scr->Root, w, h, Scr->d_depth);
+#ifdef ORIGINAL_ICONMGRPIXMAP
+    Draw3DBorder (col->pix, 0, 0, w, h, 4, cp, off, True, False);
+#else
+    Draw3DBorder (col->pix, 0, 0, w, h, 1, cp, off, True, False);
+#ifdef DO_DOT
+    Draw3DBorder (col->pix, (w / 2) - 1, (h / 2) - 1, 3, 3, 1, cp, off, True, False);
+#endif
+#endif
+    col->next = colori;
+    colori = col;
+
+    return (colori->pix);
+}
+#endif
+
 void CreateIconManagers()
 {
 	XClassHint *class; /* djhjr - 2/28/99 */
@@ -90,11 +145,13 @@ void CreateIconManagers()
     if (Scr->NoIconManagers)
 	return;
 
+/* djhjr - 10/30/02
     if (Scr->siconifyPm == None)
     {
-	Scr->siconifyPm = XCreatePixmapFromBitmapData(dpy, Scr->Root,
+	Scr->siconifyPm->pixmap = XCreatePixmapFromBitmapData(dpy, Scr->Root,
 	    (char *)siconify_bits, siconify_width, siconify_height, 1, 0, 1);
     }
+*/
 
     for (p = &Scr->iconmgr; p != NULL; p = p->next)
     {
@@ -538,9 +595,17 @@ WList *AddIconManager(tmp_win)
 
     /* djhjr - 4/19/96 */
 	/* was 'Scr->use3Diconmanagers' - djhjr - 8/11/98 */
-    if (Scr->IconMgrBevelWidth > 0) {
+/* djhjr - 10/30/02
+    if (Scr->IconMgrBevelWidth > 0)
+*/
+    {
 	if (!Scr->BeNiceToColormap) GetShadeColors (&tmp->cp);
+/* djhjr - 10/30/02
 	tmp->iconifypm = Create3DIconManagerIcon (tmp->cp);
+*/
+	tmp->iconifypm = GetImage(Scr->iconMgrIconName,
+				iconifybox_width, iconifybox_height,
+				0, tmp->cp);
     }
 
 	/* djhjr - 5/2/98 */
@@ -728,6 +793,10 @@ void RemoveIconManager(tmp_win)
 
     tmp = tmp_win->list;
 
+    /* submitted by Jonathan Paisley - 11/11/02 */
+    if (Active == tmp)
+	Active = NULL;
+
 	/*
 	 * Believe it or not, the kludge in events.c:HandleKeyPress() needs
 	 * this, or a window that's been destroyed still registers there,
@@ -735,10 +804,14 @@ void RemoveIconManager(tmp_win)
 	 *
 	 * djhjr - 6/5/98
 	 */
-#ifdef NEVER /* warps to icon managers uniquely handled in menus.c:WarpToWindow() */
+	/*
+	 * Somehwere alone the line, whatever it was got fixed, and this is
+	 * needed again - djhjr - 5/27/03
+	 */
+/*#ifdef NEVER*/ /* warps to icon managers uniquely handled in menus.c:WarpToWindow() */
 	tmp->active = FALSE;
 	tmp->iconmgr->active = NULL;
-#endif
+/*#endif*/
 
     tmp_win->list = NULL;
     ip = tmp->iconmgr;
