@@ -36,6 +36,7 @@
  * 10-Oct-90 David M. Sternlicht        Storing saved colors on root
  ***********************************************************************/
 
+#include <sys/stat.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h> /* for sleep() */
@@ -61,6 +62,10 @@
 #include <X11/Xproto.h>
 #include <X11/Xatom.h>
 #include <X11/Xmu/Error.h>
+/* djhjr - 9/14/03 */
+#ifndef NO_I18N_SUPPORT
+#include <X11/Xlocale.h>
+#endif
 
 Display *dpy;			/* which display are we talking to */
 Window ResizeWindow;		/* the window we are resizing */
@@ -73,6 +78,7 @@ ScreenInfo **ScreenList;	/* structures for each screen */
 ScreenInfo *Scr = NULL;		/* the cur and prev screens */
 int PreviousScreen;		/* last screen that we were on */
 int FirstScreen;		/* TRUE ==> first screen of display */
+Bool PrintPID = False;		/* controls PID file - djhjr - 12/2/01 */
 Bool PrintErrorMessages = False;	/* controls error messages */
 static int RedirectError;	/* TRUE ==> another window manager running */
 static int CatchRedirectError();	/* for settting RedirectError */
@@ -82,7 +88,9 @@ void InternUsefulAtoms();
 
 char Info[INFO_LINES][INFO_SIZE];		/* info strings to print */
 int InfoLines;
+
 char *InitFile = NULL;
+int parseInitFile = TRUE;	/* djhjr - 10/7/02 */
 
 Cursor UpperLeftCursor;		/* upper Left corner cursor */
 Cursor RightButt;
@@ -113,7 +121,7 @@ int JunkX;			/* junk variable */
 int JunkY;			/* junk variable */
 unsigned int JunkWidth, JunkHeight, JunkBW, JunkDepth, JunkMask;
 
-char *ProgramName;
+char *ProgramName, *PidName = "vtwm.pid"; /* PID file - djhjr - 12/2/01 */
 int Argc;
 char **Argv;
 char **Environ;
@@ -122,6 +130,11 @@ Bool RestartPreviousState = False;	/* try to restart in previous state */
 
 unsigned long black, white;
 
+/* djhjr - 9/14/03 */
+#ifndef NO_I18N_SUPPORT
+Bool use_fontset;
+#endif
+
 extern void assign_var_savecolor();
 
 /* djhjr - 4/26/99 */
@@ -158,6 +171,11 @@ main(argc, argv, environ)
 #ifndef NO_SOUND_SUPPORT
     int sound_state = 0;
 #endif
+    extern char *defTwmrc[];	/* djhjr - 10/7/02 */
+/* djhjr - 9/14/03 */
+#ifndef NO_I18N_SUPPORT
+    char *loc;
+#endif
 
     /* djhjr - 7/21/98 */
     SIGNAL_T QueueRestartVtwm();
@@ -177,9 +195,14 @@ main(argc, argv, environ)
 		if (++i >= argc) goto usage;
 		display_name = argv[i];
 		continue;
-	      case 'f':				/* -file initfile */
-		if (++i >= argc) goto usage;
-		InitFile = argv[i];
+	      case 'f':				/* -file [initfile] */
+		/* this isn't really right, but hey... - djhjr - 10/7/02 */
+		if (i + 1 < argc &&
+				(argv[i + 1][0] != '-' ||
+				(argv[i + 1][0] == '-' && !strchr("dfmpsv", argv[i + 1][1]))))
+			InitFile = argv[++i];
+		else
+			parseInitFile = FALSE;
 		continue;
 #ifndef NO_M4_SUPPORT
 	      case 'm':				/* -m4 [options] */
@@ -187,10 +210,13 @@ main(argc, argv, environ)
 		/* this isn't really right, but hey... - djhjr - 2/20/98 */
 		if (i + 1 < argc &&
 				(argv[i + 1][0] != '-' ||
-				(argv[i + 1][0] == '-' && !strchr("dfsv", argv[i + 1][1]))))
+				(argv[i + 1][0] == '-' && !strchr("dfmpsv", argv[i + 1][1]))))
 			m4_option = argv[++i];
 		continue;
 #endif
+	      case 'p':				/* -pidfile - djhjr - 12/2/01 */
+		PrintPID = True;
+		continue;
 	      case 's':				/* -single */
 		MultiScreen = FALSE;
 		continue;
@@ -207,28 +233,53 @@ main(argc, argv, environ)
       usage:
 	fprintf (stderr,
 #ifndef NO_M4_SUPPORT
-		 "usage:  %s [-d display] [-f initfile] [-m [options]] [-s] [-v]\n",
+		 "usage:  %s [-d display] [-f [initfile]] [-m [options]] [-p] [-s] [-v]\n",
 #else
-		 "usage:  %s [-d display] [-f initfile] [-s] [-v]\n",
+		 "usage:  %s [-d display] [-f [initfile]] [-p] [-s] [-v]\n",
 #endif
 		 ProgramName);
 	exit (1);
     }
 
-/* djhjr - 6/22/01 */
-#ifndef NO_SOUND_SUPPORT
-#define newhandler(sig) \
-    if (signal (sig, SIG_IGN) != SIG_IGN) (void) signal (sig, PlaySoundDone)
-#else
-#define newhandler(sig) \
-    if (signal (sig, SIG_IGN) != SIG_IGN) (void) signal (sig, Done)
+/* djhjr - 9/14/03 */
+#ifndef NO_I18N_SUPPORT
+    loc = setlocale(LC_ALL, "");
+    if (!loc || !strcmp(loc, "C") || !strcmp(loc, "POSIX") ||
+		!XSupportsLocale())
+	use_fontset = False;
+    else
+	use_fontset = True;
+
+    if (PrintErrorMessages)
+	fprintf(stderr, "%s: I18N supported, L10N %sabled\n",
+		ProgramName, (use_fontset) ? "en" : "dis");
 #endif
 
-    newhandler (SIGINT);
-    newhandler (SIGHUP);
-    newhandler (SIGQUIT);
-    newhandler (SIGTERM);
-#undef newhandler
+/* djhjr - 6/22/01 */
+#ifndef NO_SOUND_SUPPORT
+#define sounddonehandler(sig) \
+    if (signal (sig, SIG_IGN) != SIG_IGN) (void) signal (sig, PlaySoundDone)
+#else
+#define sounddonehandler(sig) \
+    if (signal (sig, SIG_IGN) != SIG_IGN) (void) signal (sig, Done)
+#endif
+#define donehandler(sig) \
+    if (signal (sig, SIG_IGN) != SIG_IGN) (void) signal (sig, Done)
+
+    sounddonehandler (SIGINT);
+    sounddonehandler (SIGHUP);
+    sounddonehandler (SIGQUIT);
+    sounddonehandler (SIGTERM);
+
+    /* djhjr - 12/2/01 */
+    donehandler (SIGABRT);
+    donehandler (SIGFPE);
+    donehandler (SIGSEGV);
+    donehandler (SIGILL);
+    donehandler (SIGTSTP);
+    donehandler (SIGPIPE);
+#undef sounddonehandler
+#undef donehandler
 
     /* djhjr - 7/31/98 */
     signal (SIGUSR1, QueueRestartVtwm);
@@ -360,6 +411,10 @@ main(argc, argv, environ)
 	Scr->SqueezeTitleL = NULL;
 	Scr->DontSqueezeTitleL = NULL;
 	Scr->WindowRingL = NULL;
+
+	/* submitted by Jonathan Paisley - 10/27/02 */
+	Scr->NoWindowRingL = NULL;
+
 	Scr->WarpCursorL = NULL;
 
 	/* djhjr - 4/22/96 */
@@ -374,6 +429,9 @@ main(argc, argv, environ)
 	/* djhjr - 5/2/98 */
 	Scr->NoBorder = NULL;
 
+	/* djhjr - 9/24/02 */
+	Scr->UsePPositionL = NULL;
+
 	/* remember to put an initialization in InitVariables also
 	 */
 
@@ -468,8 +526,15 @@ main(argc, argv, environ)
 
 	Scr->IconDirectory = NULL;
 
-	Scr->siconifyPm = None;
-	Scr->pullPm = None;
+	/* djhjr - 10/30/02 */
+	Scr->hiliteName = NULL;
+	Scr->menuIconName = TBPM_MENU;
+	Scr->iconMgrIconName = TBPM_XLOGO;
+
+/* djhjr - 10/30/02
+	Scr->siconifyPm = NULL;
+	Scr->pullPm = NULL;
+*/
 
 /* djhjr - 5/17/98 */
 #ifdef ORIGINAL_PIXMAPS
@@ -477,21 +542,14 @@ main(argc, argv, environ)
 	Scr->virtualPm = None; /* RFB PIXMAP */
 	Scr->RealScreenPm = None; /* RFB PIXMAP */
 #else /* ORIGINAL_PIXMAPS */
+	/* djhjr - 10/25/02 */
+	Scr->hiliteName = NULL;
+
 	Scr->hilitePm = NULL;
 	Scr->virtualPm = NULL;
 	Scr->realscreenPm = NULL;
 #endif /* ORIGINAL_PIXMAPS */
 
-	Scr->tbpm.xlogo = None;
-	Scr->tbpm.resize = None;
-	Scr->tbpm.question = None;
-	Scr->tbpm.menu = None;
-	Scr->tbpm.delete = None;
-
-	/* djhjr - 6/4/00 */
-	Scr->tbpm.darrow = None;
-	Scr->tbpm.rarrow = None;
-
 	if ( Scr->FirstTime )
 	{	/* retain max size on restart. */
 		Scr->VirtualDesktopMaxWidth = 0;
@@ -501,13 +559,19 @@ main(argc, argv, environ)
 	InitVariables();
 	InitMenus();
 
-	/* Parse it once for each screen. */
+	/* added this 'if (...) else' - djhjr - 10/7/02 */
+	if (!parseInitFile)
+		ParseStringList(defTwmrc);
+	else
+	{
+		/* Parse it once for each screen. */
 #ifndef NO_M4_SUPPORT
-	/* added 'm4_option' - djhjr - 2/20/99 */
-	ParseTwmrc(InitFile, display_name, m4_preprocess, m4_option);
+		/* added 'm4_option' - djhjr - 2/20/99 */
+		ParseTwmrc(InitFile, display_name, m4_preprocess, m4_option);
 #else
-	ParseTwmrc(InitFile);
+		ParseTwmrc(InitFile);
 #endif
+	}
 
 /* djhjr - 6/22/01 */
 #ifndef NO_SOUND_SUPPORT
@@ -526,10 +590,14 @@ main(argc, argv, environ)
 
 	assign_var_savecolor(); /* storeing pixels for twmrc "entities" */
 
+/* djhjr - 10/17/02 */
+#if 0
 	/* djhjr - 4/19/96 */
 	/* was 'Scr->use3Dtitles' - djhjr - 8/11/98 */
 	if (Scr->TitleBevelWidth > 0) {
+/* djhjr - 10/17/02
 	    if (Scr->TBInfo.border == -100) Scr->TBInfo.border = 0;
+*/
 
 /* djhjr - 3/12/97
 	    if (Scr->ButtonIndent  == -100) Scr->ButtonIndent  = 0;
@@ -546,17 +614,21 @@ main(argc, argv, environ)
 */
 	}
 	else {
-	    if (Scr->FramePadding  == -100) Scr->FramePadding  = 2; /* values that look */
-	    if (Scr->TitlePadding  == -100) Scr->TitlePadding  = 8; /* "nice" on */
-	    if (Scr->ButtonIndent  == -100) Scr->ButtonIndent  = 1; /* 75 and 100dpi displays */
-	    if (Scr->TBInfo.border == -100) Scr->TBInfo.border = 1;
-
 		/* djhjr - 4/5/98 */
 		Scr->SunkFocusWindowTitle = FALSE;
 	}
+#endif
+
+	/* was only if Scr->TitleBevelWidth == 0 - djhjr - 10/17/02 */
+	if (Scr->FramePadding  == -100) Scr->FramePadding  = 2; /* values that look */
+	if (Scr->TitlePadding  == -100) Scr->TitlePadding  = 8; /* "nice" on */
+	if (Scr->ButtonIndent  == -100) Scr->ButtonIndent  = 1; /* 75 and 100dpi displays */
+	if (Scr->TBInfo.border == -100) Scr->TBInfo.border = 1;
+
 	/* was 'Scr->use3D*' - djhjr - 8/11/98 */
-	if (Scr->TitleBevelWidth > 0 && !Scr->BeNiceToColormap) GetShadeColors (&Scr->TitleC);
-	if (Scr->MenuBevelWidth > 0 && !Scr->BeNiceToColormap) GetShadeColors (&Scr->MenuC);
+	/* rem'd 'Scr->*BevelWidth > 0' - djhjr - 10/30/02 */
+	if (/*Scr->TitleBevelWidth > 0 && */!Scr->BeNiceToColormap) GetShadeColors (&Scr->TitleC);
+	if (/*Scr->MenuBevelWidth > 0 && */!Scr->BeNiceToColormap) GetShadeColors (&Scr->MenuC);
 	if (Scr->MenuBevelWidth > 0 && !Scr->BeNiceToColormap) GetShadeColors (&Scr->MenuTitleC);
 	if (Scr->BorderBevelWidth > 0 && !Scr->BeNiceToColormap) GetShadeColors (&Scr->BorderColorC);
 
@@ -593,6 +665,8 @@ main(argc, argv, environ)
 	CreateGCs();
 	MakeMenus();
 
+/* djhjr - 10/18/02 */
+#if 0
 	/*
 	 * Set titlebar height from font height and padding,
 	 * then adjust to titlebutton height - djhjr - 12/10/98
@@ -608,13 +682,29 @@ main(argc, argv, environ)
 		* djhjr - 4/19/96 *
 		if (Scr->use3Dtitles) Scr->TitleHeight += 4;
 */
-		/* was 'Scr->use3Dtitles' - djhjr - 8/11/98 */
+/* djhjr - 10/18/02
+		* was 'Scr->use3Dtitles' - djhjr - 8/11/98 *
 		if (Scr->TitleBevelWidth > 0)
 			Scr->TitleHeight += 2 * Scr->TitleBevelWidth + 2;
+*/
 
 		/* make title height be odd so buttons look nice and centered */
 		if (!(Scr->TitleHeight & 1)) Scr->TitleHeight++;
 	} while ((i = InitTitlebarButtons()) > Scr->TitleHeight - Scr->FramePadding * 2);
+#else
+	/* set titlebar height to font height plus frame padding */
+	Scr->TitleHeight = Scr->TitleBarFont.height + Scr->FramePadding * 2;
+	if (!(Scr->TitleHeight & 1)) Scr->TitleHeight++;
+
+	i = InitTitlebarButtons();	/* returns the button height */
+	
+ 	/* adjust titlebar height to button height */
+	if (i > Scr->TitleHeight) Scr->TitleHeight = i + Scr->FramePadding * 2;
+	if (!(Scr->TitleHeight & 1)) Scr->TitleHeight++;
+
+	/* adjust font baseline */
+	Scr->TitleBarFont.y += ((Scr->TitleHeight - Scr->TitleBarFont.height) / 2);
+#endif
 
 	XGrabServer(dpy);
 	XSync(dpy, 0);
@@ -719,7 +809,12 @@ main(argc, argv, environ)
 					 (Visual *) CopyFromParent,
 					 valuemask, &attributes);
 
+/* djhjr - 9/14/03 */
+#ifndef NO_I18N_SUPPORT
+	Scr->SizeStringWidth = MyFont_TextWidth (&Scr->SizeFont,
+#else
 	Scr->SizeStringWidth = XTextWidth (Scr->SizeFont.font,
+#endif
 /* djhjr - 5/9/96
 					   " 8888 x 8888 ", 13);
 */
@@ -822,6 +917,32 @@ main(argc, argv, environ)
 	if (sound_state == 1) ToggleSounds();
 #endif
 
+    
+    /* write out a PID file - djhjr - 12/2/01 */
+    if (PrintPID)
+    {
+	int fd, err = 0;
+	char buf[10], *fn = malloc(HomeLen + strlen(PidName) + 2);
+
+	/* removed group and other permissions - djhjr - 10/20/02 */
+	sprintf(fn, "%s/%s", Home, PidName);
+	if ((fd = open(fn,
+		O_WRONLY|O_EXCL|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR)) != -1)
+	{
+	    sprintf(buf, "%d\n", getpid());
+	    err = write(fd, buf, strlen(buf));
+	    close(fd);
+	}
+
+	if (fd == -1 || err == -1)
+	{
+	    fprintf(stderr, "%s: cannot write to %s\n", ProgramName, fn);
+	    DoAudible();
+	}
+
+	free(fn);
+    }
+
     HandleEvents();
 
 	return (0);
@@ -864,6 +985,10 @@ void InitVariables()
     FreeList(&Scr->SqueezeTitleL);
     FreeList(&Scr->DontSqueezeTitleL);
     FreeList(&Scr->WindowRingL);
+
+    /* submitted by Jonathan Paisley - 10/27/02 */
+    FreeList(&Scr->NoWindowRingL);
+
     FreeList(&Scr->WarpCursorL);
     FreeList(&Scr->NailedDown);
     FreeList(&Scr->VirtualDesktopColorFL);
@@ -889,6 +1014,9 @@ void InitVariables()
 	/* djhjr - 5/2/98 */
 	FreeList(&Scr->NoBorder);
 
+	/* djhjr - 9/24/02 */
+	FreeList(&Scr->UsePPositionL);
+
     NewFontCursor(&Scr->FrameCursor, "top_left_arrow");
     NewFontCursor(&Scr->TitleCursor, "top_left_arrow");
     NewFontCursor(&Scr->IconCursor, "top_left_arrow");
@@ -961,14 +1089,9 @@ void InitVariables()
 #ifdef ORIGINAL_PIXMAPS
     Scr->UnknownWidth = 0;
     Scr->UnknownHeight = 0;
-#else /* ORIGINAL_PIXMAPS */
-#ifdef NO_XPM_SUPPORT
-    Scr->UnknownWidth = 0;
-    Scr->UnknownHeight = 0;
 #else
-    Scr->UnknownPm = NULL;
+    Scr->unknownName = NULL;
 #endif
-#endif /* ORIGINAL_PIXMAPS */
 
     Scr->NumAutoRaises = 0;
 /*  Scr->NoDefaults = FALSE;  */
@@ -978,6 +1101,13 @@ void InitVariables()
     Scr->FocusRoot = TRUE;
     Scr->Newest = NULL; /* PF */
     Scr->Focus = NULL;
+
+    /* djhjr - 9/10/03 */
+    Scr->IgnoreModifiers = 0;
+
+    /* djhjr - 10/16/02 */
+    Scr->WarpCentered = WARPC_OFF;
+
     Scr->WarpCursor = FALSE;
     Scr->ForceIcon = FALSE;
     Scr->NoGrabServer = FALSE;
@@ -998,6 +1128,9 @@ void InitVariables()
     /* djhjr - 10/2/01 */
     Scr->StrictIconManager = FALSE;
 
+    /* djhjr - 8/23/02 */
+    Scr->NoBorders = FALSE;
+
     Scr->NoTitlebar = FALSE;
     Scr->DecorateTransients = FALSE;
     Scr->IconifyByUnmapping = FALSE;
@@ -1006,6 +1139,7 @@ void InitVariables()
     Scr->BackingStore = TRUE;
     Scr->SaveUnder = TRUE;
     Scr->RandomPlacement = FALSE;
+    Scr->PointerPlacement = FALSE;
     Scr->OpaqueMove = FALSE;
 
 	/* djhjr - 4/6/98 */
@@ -1065,8 +1199,10 @@ void InitVariables()
     Scr->use3Dicons = FALSE;
 */
 
-	/* djhjr - 4/26/96 */
+/* obsoleted by the ":xpm:*" built-in pixmaps - djhjr - 10/26/02
+	* djhjr - 4/26/96 *
     Scr->SunkFocusWindowTitle = FALSE;
+*/
 
 	/* for rader - djhjr - 2/9/99 */
 	Scr->NoPrettyTitles = FALSE;
@@ -1197,6 +1333,9 @@ void InitVariables()
 	/* djhjr - 6/22/01 */
 	Scr->PauseOnExit = 0;
 	Scr->PauseOnQuit = 0;
+
+	/* djhjr - 11/3/03 */
+	Scr->RaiseOnStart = FALSE;
 }
 
 
@@ -1297,6 +1436,20 @@ Time time;
     SetFocus ((TwmWindow*)NULL, time);
 }
 
+/* delete the PID file - djhjr - 12/2/01 */
+void delete_pidfile()
+{
+    char *fn;
+
+    if (PrintPID)
+    {    
+	fn = malloc(HomeLen + strlen(PidName) + 2);
+	sprintf(fn, "%s/%s", Home, PidName);
+	unlink(fn);
+	free(fn);
+    }
+}
+
 
 /*
  * Exit handlers. Clean up and exit VTWM.
@@ -1327,6 +1480,9 @@ void Done()
     SetRealScreen(0,0);
     Reborder (CurrentTime);
     XCloseDisplay(dpy);
+    
+    delete_pidfile(); /* djhjr - 12/2/01 */
+
     exit(0);
 }
 #else
@@ -1335,6 +1491,9 @@ SIGNAL_T Done()
     SetRealScreen(0,0);
     Reborder (CurrentTime);
     XCloseDisplay(dpy);
+    
+    delete_pidfile(); /* djhjr - 12/2/01 */
+
     exit(0);
     SIGNAL_RETURN;
 }
@@ -1346,6 +1505,8 @@ QueueRestartVtwm()
 {
     XClientMessageEvent ev;
 
+    delete_pidfile(); /* djhjr - 12/2/01 */
+
     ev.type = ClientMessage;
     ev.window = Scr->Root;
     ev.message_type = _XA_TWM_RESTART;
